#!/usr/bin/python
# -*- coding: utf-8 -*-

#
# PANDA3D ENGINE IMPORTS
#
from panda3d.core import PandaNode

__author__ = "Fireclaw the Fox"
__license__ = """
Simplified BSD (BSD 2-Clause) License.
See License.txt or http://opensource.org/licenses/BSD-2-Clause for more info
"""


#
# CAMERA FUNCTIONS
#
class CameraFirstPerson:
    def __init__(self, parent, near=0.5, far=5000, fov=90):
        self.parent = parent
        # set the minimum and maximum view distance
        base.camLens.setNearFar(near, far)
        base.camLens.setFov(fov)
        # if this variable is set to True, the camera task will reset
        # the cursor position in one frame bevore it gets read
        # This is to fix the problem that occurs when resuming from
        # pause
        self.resetCursorOnce = False

    def startCamera(self):
        """Starts the camera module."""
        self.eyes = self.parent.exposeJoint(None, "modelRoot", "fps_eyes")
        camera.setPos(0,0,0)
        camera.setHpr(0,0,0)
        camera.reparentTo(self.eyes)
        # the cam_floater is important to trac the camera to from outside
        # the main camera classes
        self.cam_floater = self.parent.mainNode
        self.cam_floater = self.parent.mainNode.attachNewNode(PandaNode("playerCamFloater"))
        self.cam_floater.setPos(self.parent.cam_floater_pos)
        
        self.TorsorControl = self.parent.controlJoint(None,"modelRoot","neck")
        self.torsor_init_rotation = self.TorsorControl.getHpr()
        self.resetCursorOnce = True
        taskMgr.add(self.updateCamera, "task_camActualisation", priority=-4)

    def stopCamera(self):
        camera.reparentTo(render)
        taskMgr.remove("task_camActualisation")
        self.cam_floater.removeNode()

    def pauseCamera(self):
        base.win.movePointer(0, self.parent.win_width_half, self.parent.win_height_half)
        camera.reparentTo(render)
        camera.setPos(self.eyes.getPos(render))
        camera.setHpr(self.eyes.getHpr(render))
        taskMgr.remove("task_camActualisation")

    def resumeCamera(self):
        camera.setPos(0,0,0)
        camera.setHpr(0,0,0)
        camera.reparentTo(self.eyes)
        self.resetCursorOnce = True
        taskMgr.add(self.updateCamera, "task_camActualisation", priority=-4)

    def centerCamera(self):
        """This method will move the camera centered behind the player model"""
        self.TorsorControl.setHpr(self.torsor_init_rotation)
        base.win.movePointer(0, self.parent.win_width_half, self.parent.win_height_half)

    def updateCamera(self, task):
        """This function will check the min and max distance of the camera to
        the defined model and will correct the position if the cam is to close
        or to far away"""
        # check if the camera should be centered
        if self.parent.isDown(self.parent.key_center_camera):
            if base.transitions.letterboxIval:
                if not base.transitions.letterboxIval.isPlaying():
                    base.transitions.letterboxOn(0.15)
            else:
                base.transitions.letterboxOn(0.15)
            self.centerCamera()

        if base.transitions.letterbox is not None:
            if not base.transitions.letterboxIval.isPlaying() \
            and not base.transitions.letterbox.isStashed():
                base.transitions.letterboxOff(0.15)

        if not base.mouseWatcherNode.hasMouse(): return task.cont
        if self.resetCursorOnce:
            base.win.movePointer(0, self.parent.win_width_half, self.parent.win_height_half)
            self.resetCursorOnce = False
        self.pointer = base.win.getPointer(0)
        mouseX = self.pointer.getX()
        mouseY = self.pointer.getY()

        if base.win.movePointer(0, self.parent.win_width_half, self.parent.win_height_half):
            mouseMovementY = (mouseY - self.parent.win_height_half) * self.parent.mouse_speed_y
            if self.parent.mouse_invert_vertical:
                p = self.TorsorControl.getP() - mouseMovementY
            else:
                p = self.TorsorControl.getP() + mouseMovementY
            if p <-80:
                p = -80
            elif p > 90:
                p = 90
            self.TorsorControl.setP(p)

            if self.parent.state not in self.parent.ledge_grab_states:
                mouseMovementX = (mouseX - self.parent.win_width_half) * self.parent.mouse_speed_x
                if self.parent.mouse_invert_horizontal:
                    h = self.parent.mainNode.getH() + mouseMovementX
                else:
                    h = self.parent.mainNode.getH() - mouseMovementX
                if h <-360:
                    h = 360
                elif h > 360:
                    h = -360
                newHpr = self.parent.mainNode.getHpr()
                newHpr.setX(h)
                self.parent.updatePlayerHpr(newHpr)
                self.TorsorControl.setH(0)
            else:
                h = self.TorsorControl.getH() + (mouseX - self.parent.win_width_half) * self.parent.mouse_speed_x
                if h <= -50:
                    h = -50
                elif h >= 50:
                    h = 50
                self.TorsorControl.setH(h)
                

        return task.cont
